<?php

namespace App\Http\Controllers\Back\Dashboard\Catalogos;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Library\URI;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\File;
use App\Http\Requests\Back\Dashboard\Catalogos\Banners\Home\CreateRequest;
use App\Http\Requests\Back\Dashboard\Catalogos\Banners\Home\EditRequest;
use Illuminate\Database\QueryException;
use App\Models\Banners;

class BannersHomeController extends Controller {
    public function index(){
        //DB::enableQueryLog();
        $banners = Banners::where("vc_interior", "home")->orderBy("i_order", "DESC")->get();
        //dd(\DB::getQueryLog());
        return view('back.dashboard.catalogos.banners.home.index', ["banners" => $banners, "permitions" => URI::checkPermitions()]);
    }

    public function create(){
        return view('back.dashboard.catalogos.banners.home.create', ["permitions" => URI::checkPermitions()]);
    }

    public function store(CreateRequest $r){
        $response = array( 'status' => false, 'message' => "No hemos podido almacenar los datos, intentalo nuevamente." );

        try {
            $banners = new Banners;
            $banners->vc_interior = "home";
            $banners->vc_nombre = $r->nombre;
            $banners->i_order = $r->orden;
            $banners->b_status = ( $r->status == null ) ? 0 : 1;

            //Desktop
            if( $r->has("desktop") ) {
                $fName = $r->file('desktop');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $fName->move("images/catalogos/banners/home", $fileName);
                $banners->vc_img_desktop = "/images/catalogos/banners/home/" . $fileName;
            }

            //Mobile
            if( $r->has("mobile") ) {
                $fName = $r->file('mobile');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $fName->move("images/catalogos/banners/home", $fileName);
                $banners->vc_img_mobile = "/images/catalogos/banners/home/" . $fileName;
            }

            if( $banners->save() ) {
                $response = array( 
                    'status' => true, 
                    'message' => "Los datos se han almacenado correctamente.", 
                    'url' => route('bannersHome.index') 
                );
            }
        } catch ( QueryException $e ) {
            if ( strpos($e->errorInfo[2], "vc_nombre") ) {
                $error = "El campo 'Tipo' en galería es obligatorio";
            } else if ( $e->errorInfo[1] == "1062" ) {
                $error = "El producto ingresado está duplicado";
            } else {
                $error = $e->getMessage();
            }
            $response = array( 'status' => false, 'message' => $error );
        }
        return response()->json($response);
    }

    public function show($id){
        //
    }

    public function edit($id){
        $banners = Banners::where("id", base64_decode($id))->get();
        return view('back.dashboard.catalogos.banners.home.edit', ["banners" => $banners, "permitions" => URI::checkPermitions()]);
    }

    public function update(EditRequest $r, $id){
        $response = array( 'status' => false, 'message' => "No hemos podido almacenar los datos, intentalo nuevamente." );    

        try {
            $banners = Banners::where("id", base64_decode($id))->get();

            $bannersUpdate = Banners::where("id", base64_decode($id))->update([
                "vc_nombre" => $r->nombre,
                "i_order" => $r->orden,
                "b_status" => ( $r->status == null ) ? 0 : 1
            ]);

            //Desktop
            if( $r->has("desktop") ) {
                $path = ltrim($banners[0]->vc_img_desktop, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }
                $fName = $r->file('desktop');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $fName->move("images/catalogos/banners/home", $fileName);

                $bannersUpdateDesktop = Banners::where("id", base64_decode($id))->update([
                    "vc_img_desktop" => "/images/catalogos/banners/home/" . $fileName,
                ]);
            }

            //Movil
            if( $r->has("mobile") ) {
                $path = ltrim($banners[0]->vc_img_mobile, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }
                $fName = $r->file('mobile');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $fName->move("images/catalogos/banners/home", $fileName);

                $bannersUpdateMovil = Banners::where("id", base64_decode($id))->update([
                    "vc_img_mobile" => "/images/catalogos/banners/home/" . $fileName,
                ]);
            }

            $response = array( 
                'status' => true, 
                'message' => "Los datos se han almacenado correctamente.", 
                'url' => route('bannersHome.index') 
            );
        } catch ( QueryException $e ) {
            if ( strpos($e->errorInfo[2], "i_tipo") ) {
                $error = "El campo 'Tipo' en galería es obligatorio";
            } else if ( $e->errorInfo[1] == "1062" ) {
                $error = "El producto ingresado está duplicado";
            } else {
                $error = $e->getMessage();
            }
            $response = array( 'status' => false, 'message' => $error );
        }

        return response()->json($response);
    }

    public function destroy($id){
        $response = array( 'status' => false, 'message' => "No hemos podido almacenar los datos, intentalo nuevamente." );

        try {
            $banners = Banners::find(base64_decode($id));

            if( $banners != null ) {

                $path = ltrim($banners->vc_img_desktop, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }

                $path = ltrim($banners->vc_img_mobile, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }

                if( $banners->delete() ) {
                    $response = [
                        'message' => 'Se ha eliminado el registro correctamente.',
                        'status' => true,
                        'url' => route('bannersHome.index') 
                    ];
                }
            } else {
                $response = [
                    'message' => 'No hemos podido eliminar el registro, intentalo nuevamente.',
                    'status' => false
                ];
            }
        } catch ( QueryException $e ) {
            $response = array( 'status' => false, 'message' => $e->getMessage() );
        }

        return response()->json($response);
    }
}
