<?php

namespace App\Http\Controllers\Back\Dashboard\Catalogos;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Library\URI;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\File;
use App\Http\Requests\Back\Dashboard\Catalogos\Productos\CreateRequest;
use App\Http\Requests\Back\Dashboard\Catalogos\Productos\EditRequest;
use Illuminate\Database\QueryException;
use App\Models\CatProductos;
use App\Models\Productos;
use App\Models\ProductosGaleria;

class ProductosController extends Controller {
    public function index(){
        //DB::enableQueryLog();
        $productos = Productos::select("tb_productos.id", "cat_productos.vc_categoria", "tb_productos.vc_producto", "tb_productos.f_precio", "tb_productos_galleria.vc_imagen")
        ->leftJoin('tb_productos_galleria', 'tb_productos.id', '=', 'tb_productos_galleria.i_producto')
        ->join('cat_productos', 'cat_productos.id', '=', 'tb_productos.i_cat')
        ->where("tb_productos_galleria.i_tipo", "1")
        ->where("tb_productos_galleria.i_catalogo", "2")
        ->orWhereNull("tb_productos_galleria.i_tipo")
        ->whereNull("tb_productos_galleria.deleted_at")
        ->whereNull("tb_productos_galleria.deleted_at")
        ->groupBy("tb_productos.id")
        ->get();
        //dd(\DB::getQueryLog());
        return view('back.dashboard.catalogos.productos.index', ["productos" => $productos, "permitions" => URI::checkPermitions()]);
    }

    public function create(){
        $catProductos = CatProductos::get();
        return view('back.dashboard.catalogos.productos.create', ["catProductos" => $catProductos, "permitions" => URI::checkPermitions()]);
    }

    public function store(CreateRequest $r){
        $response = array( 'status' => false, 'message' => "No hemos podido almacenar los datos, intentalo nuevamente." );

        try {
            $productos = new Productos;
            $productos->i_cat = $r->categoria;
            $productos->vc_codigo_articulo = $r->codigoProducto;
            $productos->vc_producto = $r->producto;
            $productos->vc_url = $r->url;
            $productos->tx_descripcion = ( $r->desc !== null ) ? $r->desc : '';
            $productos->f_largo = $r->largo;
            $productos->f_alto = $r->alto;
            $productos->f_ancho = $r->ancho;
            $productos->f_peso = $r->peso;
            $productos->f_precio = number_format($r->precio, 2, '.', '');
            $productos->i_descuento =( $r->descuento == "" ) ? 0 : $r->descuento;
            $productos->b_status = ( $r->status == null ) ? 0 : 1;

            if ( $productos->save() ) {
                //Files
                if( $r->has("imagen") ) {
                    $arrayGaleria = $r->file('imagen');
                    foreach ( $arrayGaleria as $key => $item ) {
                        //dd( $productos->id, $r->tipo[$key], $r->post() );
                        $galeria = new ProductosGaleria;
                        $galeria->i_producto = $productos->id;
                        $galeria->i_catalogo = 2;
                        $galeria->i_tipo = $r->tipo[$key];
                        $galeria->vc_titulo = $r->tituloImagen[$key];
                        $galeria->tx_descripcion = $r->descImagen[$key];
                        $galeria->b_status = 1;

                        $fName = $item;
                        $fileName = MD5($item . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                        $item->move("images/catalogos/productos", $fileName);
                        $galeria->vc_imagen = "/images/catalogos/productos/" . $fileName;

                        $galeria->save();
                    }
                }
            }

            $response = array( 
                'status' => true, 
                'message' => "Los datos se han almacenado correctamente.", 
                'url' => route('productos.index') 
            );
        } catch ( QueryException $e ) {
           if ( strpos($e->errorInfo[2], "i_tipo") ) {
                $error = "El campo 'Tipo' en galería es obligatorio";
            } else if ( $e->errorInfo[1] == "1062" ) {
                $error = "El producto ingresado está duplicado";
            } else {
                $error = $e->getMessage();
            }
            $response = array( 'status' => false, 'message' => $error );
        }
        return response()->json($response);
    }

    public function show($id){
        //
    }

    public function edit($id){
        $productos = Productos::where("id", base64_decode($id))->get();
        $catProductos = CatProductos::get();
        $catGaleria = ProductosGaleria::where("i_producto", $productos[0]->id)->where("i_catalogo",2)->get();
        return view('back.dashboard.catalogos.productos.edit', ["productos" => $productos, "catProductos" => $catProductos, "catGaleria" => $catGaleria, "permitions" => URI::checkPermitions()]);
    }

    public function update(EditRequest $r, $id){
        $response = array( 'status' => false, 'message' => "No hemos podido almacenar los datos, intentalo nuevamente." );    

        try {
            $productos = Productos::where("id", base64_decode($id))->get();
            $galeria = Productos::where("id", base64_decode($id))->get();

            $productosUpdate = Productos::where("id", base64_decode($id))->update([
                "i_cat" => $r->categoria,
                "vc_producto" => $r->producto,
                "vc_codigo_articulo" => $r->codigoProducto,
                "vc_url" => $r->url,
                "tx_descripcion" => ( $r->desc !== null ) ? $r->desc : '',
                "f_largo" => $r->largo,
                "f_alto" => $r->alto,
                "f_ancho" => $r->ancho,
                "f_peso" => $r->peso,
                "f_precio" => number_format($r->precio, 2, '.', ''),
                "i_descuento" => ( $r->descuento == "" ) ? 0 : $r->descuento,
                "b_status" => ( $r->status == null ) ? 0 : 1
            ]);

            //GaleriaToUpdate
            if ( $r->idImagenToUpdate != null ){
                foreach( $r->idImagenToUpdate as $key => $item ) {
                    $galeriaDestroy = ProductosGaleria::where("id", $item)->get();

                    if( isset($r->file('imagenToUpdate')[$key]) ) {
                        $path = ltrim($galeriaDestroy[0]->vc_imagen, '/');
                        if ( File::exists($path) ) {
                            unlink( $path );
                        }

                        $fName = $r->file('imagenToUpdate')[$key];
                        $fileName = MD5($r->file('imagenToUpdate')[$key] . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                        $r->file('imagenToUpdate')[$key]->move("images/catalogos/productos", $fileName);
                        $updateGaleria = ProductosGaleria::where("id", $item)->update([ "vc_imagen" => "/images/catalogos/productos/" . $fileName ]);

                        ProductosGaleria::where("id", $item)
                        ->update([
                            "i_tipo" => $r->tipoToUpdate[$key],
                            "vc_titulo" => $r->tituloImagenToUpdate[$key],
                            "tx_descripcion" => $r->descImagenToUpdate[$key]
                        ]);
                    } else {
                        ProductosGaleria::where("id", $item)
                        ->update([
                            "i_tipo" => $r->tipoToUpdate[$key],
                            "vc_titulo" => $r->tituloImagenToUpdate[$key],
                            "tx_descripcion" => $r->descImagenToUpdate[$key]
                        ]);
                    }
                }
            }

            //Galeria
            if( $r->has("imagen") ) {
                $arrayGaleria = $r->file('imagen');
                foreach ( $arrayGaleria as $key => $item ) {
                    $galeria = new ProductosGaleria;
                    $galeria->i_producto = base64_decode($id);
                    $galeria->i_catalogo = 2;
                    $galeria->i_tipo = $r->tipo[$key];
                    $galeria->vc_titulo = $r->tituloImagen[$key];
                    $galeria->tx_descripcion = $r->descImagen[$key];
                    $galeria->b_status = 1;

                    $fName = $item;
                    $fileName = MD5($item . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                    $item->move("images/catalogos/productos", $fileName);
                    $galeria->vc_imagen = "/images/catalogos/productos/" . $fileName;

                    $galeria->save();
                }
            }

            $response = array( 
                'status' => true, 
                'message' => "Los datos se han almacenado correctamente.", 
                'url' => route('productos.index') 
            );
        } catch ( QueryException $e ) {
            if ( strpos($e->errorInfo[2], "i_tipo") ) {
                $error = "El campo 'Tipo' en galería es obligatorio";
            } else if ( $e->errorInfo[1] == "1062" ) {
                $error = "El producto ingresado está duplicado";
            } else {
                $error = $e->getMessage();
            }
            $response = array( 'status' => false, 'message' => $error );
        }

        return response()->json($response);
    }

    public function destroy($id){
        $response = array( 'status' => false, 'message' => "No hemos podido almacenar los datos, intentalo nuevamente." );

        try {
            $productos = Productos::find(base64_decode($id));
            $galeria = ProductosGaleria::where("i_producto", $productos->id)->where("i_catalogo", 2)->get();
            if( $productos != null ) {
                if( $productos->delete() ) {
                    if ( count($galeria) > 0 ){
                        foreach( $galeria as $item ) {
                            $path = ltrim($item->vc_imagen, '/');
                            if ( File::exists($path) ) {
                                unlink( $path );
                            }
                        }
                    }

                    $response = [
                        'message' => 'Se ha eliminado el registro correctamente.',
                        'status' => true,
                        'url' => route('productos.index') 
                    ];
                }
            } else {
                $response = [
                    'message' => 'No hemos podido eliminar el registro, intentalo nuevamente.',
                    'status' => false
                ];
            }
        } catch ( QueryException $e ) {
            $response = array( 'status' => false, 'message' => $e->getMessage() );
        }

        return response()->json($response);
    }
}
