<?php

namespace App\Http\Controllers\Back\Dashboard\Catalogos\PuertasAMedida;

use App\Http\Controllers\Controller;
use App\Library\URI;
use Illuminate\Http\Request;
use App\Http\Requests\Back\Dashboard\Catalogos\PuertasAMedida\Elementos\CreateRequest;
use App\Http\Requests\Back\Dashboard\Catalogos\PuertasAMedida\Elementos\EditRequest;
use Illuminate\Support\Facades\File;
use App\Models\Elementos;
use App\Models\CatElementos;

class ElementosController extends Controller {
    public function index(){
        //DB::enableQueryLog();
        $data = Elementos::select("tb_elementos.id", "cat_elementos.vc_elemento as tipo", "tb_elementos.vc_nombre as nombre", "tb_elementos.b_status", "tb_elementos.vc_img_batiente", "tb_elementos.vc_img_doble", "tb_elementos.vc_img_pivote", "tb_elementos.vc_img_batiente_fijo", "tb_elementos.vc_img_preview")
        ->join("cat_elementos", "cat_elementos.id", "=", "tb_elementos.id_elemento_tipo")
        ->get();
        //dd(\DB::getQueryLog());
        return view('back.dashboard.puertasmedida.elementos.index', ["data" => $data, "permitions" => URI::checkPermitions()]);
    }

    public function create(){
        $catElementos = CatElementos::get();
        return view('back.dashboard.puertasmedida.elementos.create', ["catElementos" => $catElementos, "permitions" => URI::checkPermitions()]);
    }

    public function store(CreateRequest $r){
        $response = array( 'status' => false, 'message' => "No hemos podido almacenar los datos, intentalo nuevamente." );

        try {
            $elementos = new Elementos;
            $elementos->id_elemento_tipo = $r->tipoElemento;
            $elementos->vc_nombre = $r->name;
            $elementos->f_precio_batiente = number_format($r->precio_batiente, 2, '.', '');
            $elementos->f_precio_doble = number_format($r->precio_doble, 2, '.', '');
            $elementos->f_precio_pivote = number_format($r->precio_pivote, 2, '.', '');
            $elementos->f_precio_batiente_fijo = number_format($r->precio_batiente_fijo, 2, '.', '');
            $elementos->b_status = ( $r->status == null ) ? 0 : 1;

            //Sencillo
            if( $r->has("pivote") ) {
                $fName = $r->file('pivote');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $fName->move("images/catalogos/puertasamedida/elementos", $fileName);
                $elementos->vc_img_pivote = "/images/catalogos/puertasamedida/elementos/" . $fileName;
            }

            //Doble
            if( $r->has("doble") ) {
                $fName = $r->file('doble');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $fName->move("images/catalogos/puertasamedida/elementos", $fileName);
                $elementos->vc_img_doble = "/images/catalogos/puertasamedida/elementos/" . $fileName;
            }

            //Batiente
            if( $r->has("batiente") ) {
                $fName = $r->file('batiente');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $fName->move("images/catalogos/puertasamedida/elementos", $fileName);
                $elementos->vc_img_batiente = "/images/catalogos/puertasamedida/elementos/" . $fileName;
            }

            //Batiente Fijo
            if( $r->has("batiente_fijo") ) {
                $fName = $r->file('batiente_fijo');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $fName->move("images/catalogos/puertasamedida/elementos", $fileName);
                $elementos->vc_img_batiente_fijo = "/images/catalogos/puertasamedida/elementos/" . $fileName;
            }

            //Preview
            if( $r->has("preview") ) {
                $fName = $r->file('preview');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $fName->move("images/catalogos/puertasamedida/elementos", $fileName);
                $elementos->vc_img_preview = "/images/catalogos/puertasamedida/elementos/" . $fileName;
            }

            if ( $elementos->save() ) {
                $response = array( 
                    'status' => true, 
                    'message' => "Los datos se han almacenado correctamente.", 
                    'url' => route('elementos.index') 
                );
            }

        } catch ( QueryException $e ) {
           if ( $e->errorInfo[1] == "1062" ) {
                $error = "El producto ingresado está duplicado";
            } else {
                $error = $e->getMessage();
            }
            $response = array( 'status' => false, 'message' => $error );
        }
        return response()->json($response);
    }

    public function show($id){
        //
    }

    public function edit($id){
        $data = Elementos::where("id", base64_decode($id))->get();
        $catElementos = CatElementos::get();
        return view('back.dashboard.puertasmedida.elementos.edit', ["data" => $data, "catElementos" => $catElementos, "permitions" => URI::checkPermitions()]);
    }

    public function update(EditRequest $r, $id){
        $response = array( 'status' => false, 'message' => "No hemos podido almacenar los datos, intentalo nuevamente." );    

        try {

            $check = Elementos::where("id", base64_decode($id))->get();
            
            $elementosUpdate = Elementos::where("id", base64_decode($id))->update([
                "id_elemento_tipo" => $r->tipoElemento,
                "vc_nombre" => $r->name,
                "f_precio_batiente" => number_format($r->precio_batiente, 2, '.', ''),
                "f_precio_doble" => number_format($r->precio_doble, 2, '.', ''),
                "f_precio_pivote" => number_format($r->precio_pivote, 2, '.', ''),
                "f_precio_batiente_fijo" => number_format($r->precio_batiente_fijo, 2, '.', ''),
                "b_status" => ( $r->status == null ) ? 0 : 1
            ]);


            //Sencillo
            if( $r->has("batiente") ) {
                $path = ltrim($check[0]->vc_img_batiente, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }
                $fName = $r->file('batiente');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $fName->move("images/catalogos/puertasamedida/elementos", $fileName);

                $elementosUpdate = Elementos::where("id", base64_decode($id))->update([
                    "vc_img_batiente" => "/images/catalogos/puertasamedida/elementos/" . $fileName,
                ]);
            }

            //Doble
            if( $r->has("doble") ) {
                $path = ltrim($check[0]->vc_img_doble, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }
                $fName = $r->file('doble');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $fName->move("images/catalogos/puertasamedida/elementos", $fileName);

                $elementosUpdate = Elementos::where("id", base64_decode($id))->update([
                    "vc_img_doble" => "/images/catalogos/puertasamedida/elementos/" . $fileName,
                ]);
            }

            //Batiente
            if( $r->has("pivote") ) {
                $path = ltrim($check[0]->vc_img_pivote, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }
                $fName = $r->file('pivote');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $fName->move("images/catalogos/puertasamedida/elementos", $fileName);

                $elementosUpdate = Elementos::where("id", base64_decode($id))->update([
                    "vc_img_pivote" => "/images/catalogos/puertasamedida/elementos/" . $fileName,
                ]);
            }

            //Batiente Fijo
            if( $r->has("batiente_fijo") ) {
                $path = ltrim($check[0]->vc_img_batiente_fijo, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }
                $fName = $r->file('batiente_fijo');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $fName->move("images/catalogos/puertasamedida/elementos", $fileName);

                $elementosUpdate = Elementos::where("id", base64_decode($id))->update([
                    "vc_img_batiente_fijo" => "/images/catalogos/puertasamedida/elementos/" . $fileName,
                ]);
            }

            //Preview
            if( $r->has("preview") ) {
                $path = ltrim($check[0]->vc_img_preview, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }
                $fName = $r->file('preview');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $fName->move("images/catalogos/puertasamedida/elementos", $fileName);

                $elementosUpdate = Elementos::where("id", base64_decode($id))->update([
                    "vc_img_preview" => "/images/catalogos/puertasamedida/elementos/" . $fileName,
                ]);
            }

            $response = array( 
                'status' => true, 
                'message' => "Los datos se han almacenado correctamente.", 
                'url' => route('elementos.index') 
            );
        } catch ( QueryException $e ) {
            if ( strpos($e->errorInfo[2], "i_tipo") ) {
                $error = "El campo 'Tipo' en galería es obligatorio";
            } else if ( $e->errorInfo[1] == "1062" ) {
                $error = "El producto ingresado está duplicado";
            } else {
                $error = $e->getMessage();
            }
            $response = array( 'status' => false, 'message' => $error );
        }

        return response()->json($response);
    }

    public function destroy($id){
        $response = array( 'status' => false, 'message' => "No hemos podido almacenar los datos, intentalo nuevamente." );

        try {
            $elementos = Elementos::find(base64_decode($id));

            if( $elementos != null ) {

                $path = ltrim($elementos->vc_img_sencillo, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }
                $path = ltrim($elementos->vc_img_doble, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }
                $path = ltrim($elementos->vc_img_batiente, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }
                $path = ltrim($elementos->vc_img_batiente_fijo, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }
                $path = ltrim($elementos->vc_img_preview, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }

                if( $elementos->delete() ) {
                    $response = [
                        'message' => 'Se ha eliminado el registro correctamente.',
                        'status' => true,
                        'url' => route('elementos.index') 
                    ];
                }
            } else {
                $response = [
                    'message' => 'No hemos podido eliminar el registro, intentalo nuevamente.',
                    'status' => false
                ];
            }
        } catch ( QueryException $e ) {
            $response = array( 'status' => false, 'message' => $e->getMessage() );
        }

        return response()->json($response);
    }
}
