<?php

use Illuminate\Support\Facades\Route;
use Illuminate\Support\Facades\Auth;
use App\Http\Controllers\Back\AccountController;
use App\Http\Controllers\Back\Dashboard\DashboardController;
use App\Http\Controllers\Back\Dashboard\System\UserController;
use App\Http\Controllers\Back\Dashboard\System\ProfileController;
use App\Http\Controllers\Back\Dashboard\Catalogos\PuertasController;
use App\Http\Controllers\Back\Dashboard\Catalogos\ProductosController;
use App\Http\Controllers\Back\Dashboard\Catalogos\BannersHomeController;
use App\Http\Controllers\Back\Dashboard\Catalogos\PuertasAMedida\ElementosController;
use App\Http\Controllers\Back\Dashboard\Catalogos\PuertasAMedida\FijosController;

use App\Models\Compras;
use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\SMTP;
use PHPMailer\PHPMailer\Exception;

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| contains the "web" middleware group. Now create something great!
|
*/

//CMS
Route::group(['prefix' => 'cms'], function () {
    Route::get('/', [AccountController::class, 'getHome'])->name('login');
    Route::group(['prefix' => 'account'], function () {
        Route::get('logout', [AccountController::class, 'getLogout'])->name('admin_logout');
        Route::post('login', [AccountController::class, 'postLogin'])->name('admin_login');
        Route::post('password/reset', [AccountController::class, 'postReset'])->name('password.reset');
    });
    Route::group(['middleware' => ['auth']], function () {
        Route::group(['prefix' => 'dashboard'], function () {

            Route::get('/', [DashboardController::class, 'index']);

            Route::group(['prefix' => 'system'], function () {
                Route::group(['prefix' => 'user'], function () {
                    Route::get('/', [UserController::class, 'index'])->name('user_index');
                    Route::get('create', [UserController::class, 'create'])->name('user_create_get');
                    Route::post('store', [UserController::class, 'store'])->name('user_create');
                    Route::get('edit/{id}', [UserController::class, 'edit'])->name('user_edit');
                    Route::post('update/{id}', [UserController::class, 'update'])->name('user_update');
                    Route::get('delete/{id}', [UserController::class, 'destroy'])->name('user_delete');
                    Route::get('edit_unique/{id}', [UserController::class, 'edit_unique'])->name('user_unique_edit');
                    Route::post('update_unique/{id}', [UserController::class, 'update_unique'])->name('user_unique_update');
                });
                Route::group(['prefix' => 'profile'], function () {
                    Route::get('/', [ProfileController::class, 'index'])->name('profile_index');
                    Route::get('create', [ProfileController::class, 'create'])->name('profile_create');
                    Route::post('store', [ProfileController::class, 'store'])->name('profile.store');
                    Route::get('edit/{id}', [ProfileController::class, 'edit'])->name('profile.edit');
                    Route::post('update', [ProfileController::class, 'update'])->name('profile.update');
                    Route::get('getData', [ProfileController::class, 'getData'])->name('profile.data');
                    Route::get('view/{id}', [ProfileController::class, 'show'])->name('profile.view');
                    Route::get('delete/{id}', [ProfileController::class, 'destroy'])->name('profile.delete');
                });
            });

            Route::group(['prefix' => 'banners'], function () {
                Route::group(['prefix' => 'home'], function () {
                    Route::get('/', [BannersHomeController::class, 'index'])->name('bannersHome.index');
                    Route::get('create', [BannersHomeController::class, 'create'])->name('bannersHome.create');
                    Route::post('store', [BannersHomeController::class, 'store'])->name('bannersHome.store');
                    Route::get('edit/{id}', [BannersHomeController::class, 'edit'])->name('bannersHome.edit');
                    Route::post('update/{id}', [BannersHomeController::class, 'update'])->name('bannersHome.update');
                    Route::get('delete/{id}', [BannersHomeController::class, 'destroy'])->name('bannersHome.delete');
                });
            });

            Route::group(['prefix' => 'catalogos'], function () {
                Route::group(['prefix' => 'puertas'], function () {
                    Route::get('/', [PuertasController::class, 'index'])->name('puertas.index');
                    Route::get('create', [PuertasController::class, 'create'])->name('puertas.create');
                    Route::post('store', [PuertasController::class, 'store'])->name('puertas.store');
                    Route::get('edit/{id}', [PuertasController::class, 'edit'])->name('puertas.edit');
                    Route::post('update/{id}', [PuertasController::class, 'update'])->name('puertas.update');
                    Route::get('delete/{id}', [PuertasController::class, 'destroy'])->name('puertas.delete');
                });

                Route::group(['prefix' => 'productos'], function () {
                    Route::get('/', [ProductosController::class, 'index'])->name('productos.index');
                    Route::get('create', [ProductosController::class, 'create'])->name('productos.create');
                    Route::post('store', [ProductosController::class, 'store'])->name('productos.store');
                    Route::get('edit/{id}', [ProductosController::class, 'edit'])->name('productos.edit');
                    Route::post('update/{id}', [ProductosController::class, 'update'])->name('productos.update');
                    Route::get('delete/{id}', [ProductosController::class, 'destroy'])->name('productos.delete');
                });
            });

            Route::group(['prefix' => 'puertas-a-medida'], function () {
                Route::group(['prefix' => 'elementos'], function () {
                    Route::get('/', [ElementosController::class, 'index'])->name('elementos.index');
                    Route::get('create', [ElementosController::class, 'create'])->name('elementos.create');
                    Route::post('store', [ElementosController::class, 'store'])->name('elementos.store');
                    Route::get('edit/{id}', [ElementosController::class, 'edit'])->name('elementos.edit');
                    Route::post('update/{id}', [ElementosController::class, 'update'])->name('elementos.update');
                    Route::get('delete/{id}', [ElementosController::class, 'destroy'])->name('elementos.delete');
                });

                Route::group(['prefix' => 'fijos'], function () {
                    Route::get('/', [FijosController::class, 'index'])->name('fijos.index');
                    Route::get('create', [FijosController::class, 'create'])->name('fijos.create');
                    Route::post('store', [FijosController::class, 'store'])->name('fijos.store');
                    Route::get('edit/{id}', [FijosController::class, 'edit'])->name('fijos.edit');
                    Route::post('update/{id}', [FijosController::class, 'update'])->name('fijos.update');
                    Route::get('delete/{id}', [FijosController::class, 'destroy'])->name('fijos.delete');
                });
            });
        });
    });
});

//WEB
Route::get('/', function () {
    return redirect("/cms");
});

/**********/
/* LOGOUT */
/**********/
    Route::get('/logout', function() {
        Auth::logout();
        Session::flush();
        return redirect('/');
    });

/***********/
/* MAILING */
/***********/
    Route::post('/sendContacto', 'SendController@send');

/********/
/* TEST */
/********/

Route::get('/clear-cache', function() {
    $exitCode = Artisan::call('cache:clear');
    $exitCode = Artisan::call('config:cache');
    return 'DONE'; //Return anything
});

Route::get('/compra', function() {
    $response["error"] = true;
    $response["message"] = "No se pudo enviar el correo de confirmación de compra.";

    $idTest = 38;

    $toMail = Compras::where("id", $idTest)->get();

    $noCompra = "P-".str_pad($idTest,8,"0",STR_PAD_LEFT);
    $items = json_decode($toMail[0]['tx_resumen'], true);
    $datos["nombre"] = $toMail[0]["vc_nombre"];
    $datos["apellido"] = $toMail[0]["vc_apellido"];
    $datos["correo"] = $toMail[0]["vc_email"];
    $datos["telefono"] = $toMail[0]["vc_telefono"];
    $datos["direccion"] = $toMail[0]["vc_direccion"];
    $datos["colonia"] = $toMail[0]["vc_colonia"];
    $datos["ciudad"] = $toMail[0]["vc_ciudad"];
    $datos["estado"] = $toMail[0]["vc_estado"];
    $datos["cp"] = $toMail[0]["i_cp"];
    $datos["last4"] = $toMail[0]["vc_last4"];
    $resumen = array(
        "subtotal" => $toMail[0]['f_subtotal'], 
        "entrega" => $toMail[0]['f_entrega'], 
        "promociones" => $toMail[0]['f_promociones'], 
        "total" => $toMail[0]['f_total'] 
    );
    return view('mails/compra-eversafe',compact( 'noCompra', 'items', 'resumen', 'datos' ));
    /*

    $view = view('mails/compra',compact( 'noCompra', 'items', 'resumen' ))->render();

    $mail = new PHPMailer(true);
    $mail->IsSMTP(); // enable SMTP

    $mail->SMTPDebug = 0;
    $mail->SMTPAuth = true;
    $mail->SMTPSecure = 'tls';
    $mail->Host = "smtp.gmail.com";
    $mail->Port = 587;
    $mail->IsHTML(true);
    $mail->Username = "mauricio.salinas@metodika.mx";
    $mail->Password = "qjfkhfzgyjeyezwv";
    $mail->SetFrom("mauricio.salinas@metodika.mx", "Metodika");
    $mail->addReplyTo("mauricio.salinas@metodika.mx");
    $mail->Subject = "Eversafe Contacto";
    $mail->Body = $view;
    $mail->AddAddress("mauricio.salinas@metodika.mx", "Metodika");
    //$mail->addBCC('leads@metodika.mx', 'Leads');
    //$mail->addBCC('mauricio.salinas@metodika.mx', 'Mauricio');
    //$mail->addBCC('carlos.betancourt@metodika.mx', 'Carlos');

    if( $mail->Send() ) {
        $response["error"] = false;
        $response["message"] = "Hemos enviado un correo con la confirmación de tu compra.";
    } else {
        $response["error"] = true;
        $response["message"] = "No hemos podido enviar el correo de confirmación, por favor contáctanos.";
    }

    return response()
        ->json([ 
            'error' => $response["error"], 
            'message' => $response["message"],
            'data' => "Correo enviado"
        ], 200)
        ->header("Access-Control-Allow-Origin", "*");
    */
});

Route::get('/envio', function() {
    $response["error"] = true;
    $response["message"] = "No se pudo enviar el correo de confirmación de compra.";

    $guia = "0000001";
    $items = array(
        array(
            "productos" => 
            array(
                array(
                    "cantidad" => 1,
                    "nombre" => "Puerta de Seguridad Cartagena",
                ),  
                array(
                    "cantidad" => 1,
                    "nombre" => "Cerradura Smart Home"
                )
            )
        ), 
        array(
            "resumen" => 
            array(
                "subtotal" => "00,000.00",
                "entrega" => "000.00",
                "total" => "000.00"
            )
        )
    );

    $view = view('mails/envio',compact( 'guia', 'items' ))->render();

    $mail = new PHPMailer(true);
    $mail->IsSMTP(); // enable SMTP

    $mail->SMTPDebug = 0;
    $mail->SMTPAuth = true;
    $mail->SMTPSecure = 'tls';
    $mail->Host = "mail.eversafe.com.mx";
    $mail->Port = 587;
    $mail->IsHTML(true);
    $mail->CharSet = 'UTF-8';
    $mail->Username = "e-commerce@eversafe.com.mx";
    $mail->Password = "uM2e%Zpyjd+";
    $mail->SetFrom("e-commerce@eversafe.com.mx", "Eversafe");
    $mail->addReplyTo("e-commerce@eversafe.com.mx");

    $mail->Subject = "Eversafe Contacto";
    $mail->Body = $view;
    $mail->AddAddress("mauricio.salinas@metodika.mx", "Metodika");
    //$mail->addBCC('leads@metodika.mx', 'Leads');
    //$mail->addBCC('mauricio.salinas@metodika.mx', 'Mauricio');
    //$mail->addBCC('carlos.betancourt@metodika.mx', 'Carlos');

    if( $mail->Send() ) {
        $response["error"] = false;
        $response["message"] = "Hemos enviado un correo con la confirmación de tu compra.";
    } else {
        $response["error"] = true;
        $response["message"] = "No hemos podido enviar el correo de confirmación, por favor contáctanos.";
    }

    return response()
        ->json([ 
            'error' => $response["error"], 
            'message' => $response["message"],
            'data' => "Correo enviado"
        ], 200)
        ->header("Access-Control-Allow-Origin", "*");
});
